-- dataMine Plugin for Vera
-- (c) Chris Jackson

module("L_DataMine1", package.seeall)


-- port_3480/data_request?id=user_data
-- Returns configuration array


-- Service ID strings used by this device.
SERVICE_ID = "urn:cd-jackson-com:serviceId:DataMine1"

local DATAMINE_CONFIG   = "dataMineConfig.json"
local DATAMINE_LOG_NAME = "dataMine: "
local DATAMINE_LOG_DIR  = "/dataMine/"

local FIRST_YEAR     = 1104516000
local LOGTIME_DAILY  = 31536000
local LOGTIME_HOURLY = 2419200
local LOGTIME_RAW    = 604800

local INVALID_VALUE = -9999989.1
local dataDir = nil
local LogVariables
local Debug = 0
local ChannelCnt = 0
local ChannelRec = 0

-- Run once at Luup engine startup.
function initialise(lul_device)
	luup.log(DATAMINE_LOG_NAME.."Initialising Data Mining System")

	-- Help prevent race condition
	luup.io.intercept()

	-- The data directory is stored as a service variable to allow it to be
	-- read out before the config file.
	dataDir = luup.variable_get(SERVICE_ID, "DataDirectory", lul_device)
	if(dataDir == nil) then
		dataDir = DATAMINE_LOG_DIR
		luup.variable_set(SERVICE_ID, "DataDirectory", DATAMINE_LOG_DIR, lul_device)
	end

	local logfile = dataDir .. DATAMINE_CONFIG

	ChannelCnt = 0
	ChannelRec = 0

	local inf = io.open(logfile, 'r')
	if(inf == nil) then
		luup.log(DATAMINE_LOG_NAME .. "Unable to open config file for read :: " .. logfile)
	else
		local line = inf:read("*all")
		if(line ~= nil) then
			--luup.log(DATAMINE_LOG_NAME .. "CONFIG: " .. line)
			json = require("json")

			LogVariables = json.decode(line)
			if(LogVariables == nil) then
				luup.log(DATAMINE_LOG_NAME .. "Config file not decoded")
			elseif(LogVariables.Variables == nil) then
				luup.log(DATAMINE_LOG_NAME .. "Config file not decoded - no variables")
			else
				for k,v in pairs (LogVariables.Variables) do
					ChannelCnt = ChannelCnt + 1
					if(v.Logging == 1) then
						ChannelRec = ChannelRec + 1
						luup.variable_watch('watchVariable', v.Service, v.Variable, tonumber(v.Device))
						luup.log(DATAMINE_LOG_NAME.."Watching: D["..tonumber(v.Device).."] S["..v.Service.."] V["..v.Variable.."]")
						if(v.LastHistory == nil) then
							v.LastHistory = FIRST_YEAR
						end
					end
				end
			end
		end
	end

	luup.variable_set(SERVICE_ID, "ChannelCnt", ChannelCnt, lul_device)
	luup.variable_set(SERVICE_ID, "ChannelRec", ChannelRec, lul_device)

	if(LogVariables == nil) then
		luup.log(DATAMINE_LOG_NAME .. "Reinitialising configuration structure")
		LogVariables = {}
		LogVariables.Variables = {}
	end

	local f=io.open("/www/cmh/plugins/dataMine/flotr2.min.js","r")
	if f~=nil then
		io.close(f)
	else
		luup.log(DATAMINE_LOG_NAME.."flotr2.min.js does not exist")
	end
	local f=io.open("/www/cmh/plugins/dataMine/flotr2.ie.min.js","r")
	if f~=nil then
		io.close(f)
	else
		luup.log(DATAMINE_LOG_NAME.."flotr2.ie.min.js does not exist")
	end

	-- Register handlers to serve the JSON data
	luup.register_handler("incomingCtrl", "dmCtrl")
	luup.register_handler("incomingData", "dmData")
	luup.register_handler("incomingList", "dmList")

	--
	--luup.call_delay('dailyHistory', 30, "", 1)
	-- Startup is done.
	luup.log(DATAMINE_LOG_NAME .. "Startup complete")
end

-- Watch Callback
function watchVariable(lul_device, lul_service, lul_variable, lul_value_old, lul_value_new)
	-- Calculate the week number
	local WeekNum = math.floor(os.time() / LOGTIME_RAW)
	local logfile  = nil
	local varRef   = nil

	for k,v in pairs (LogVariables.Variables) do
		if(v.Device==tonumber(lul_device) and v.Service==lul_service and v.Variable==lul_variable) then
			if(v.Logging == 0) then
				-- Return if logging is disabled for this variable
				return
			end
			if(v.FirstRec == 0) then
				json = require("json")

				v.FirstRec = os.time()
				saveConfig()
			end

			-- logfile
			logfile = dataDir .. v.Archive .. " [R" .. WeekNum .. "].txt"

			-- save reference
			varRef = v
			break
		end
	end

	if(logfile == nil) then
		luup.log(DATAMINE_LOG_NAME .. "Didn't find variable!")
		return
	end

--	luup.log(DATAMINE_LOG_NAME .. varRef .. "---" .. varRef.Name)

	if(tonumber(lul_value_new) == nil) then
		if(varRef.Lookup[lul_value_new] == nil) then
			local	max = 0
			for k,v in pairs (varRef.Lookup) do
				if(v > max) then
					max = v
				end
			end
--			luup.log(DATAMINE_LOG_NAME .. "---" .. varRef.Name .. "---" .. max)
			varRef.Lookup[lul_value_new] = max + 1
			varRef.Alpha = 1
			saveConfig()
		end
	end


--	luup.log ((DATAMINE_LOG_NAME .. logfile .. " WEEK:"..WeekNum.." watchVarVariable(%s/%s/%s/%s/%s)"):format (tostring(lul_device), lul_service, lul_variable, tostring(lul_value_old), tostring(lul_value_new)))

	local outf = io.open(logfile, 'a')
	if(outf == nil) then
		luup.log(DATAMINE_LOG_NAME .. "Unable to open file for write " .. logfile)
	else
		outf:write(os.time() .. ',' .. tostring(lul_value_new) .. '\n')
		outf:close()
	end
end

-- Get a list of channels that can be graphed
function incomingList(lul_request, lul_parameters, lul_outputformat)
--	luup.log(DATAMINE_LOG_NAME .. "incomingList: " .. lul_request)

	local lul_html = '{"list":['

	local first = 1;
	for k,v in pairs (LogVariables.Variables) do
		if(first == 0) then
			lul_html = lul_html .. ","
		end
		first = 0
		lul_html = lul_html .. '{"name":"'..v.Name..'","id":'..k..',"logging":'..v.Logging..',"type":'..v.Type..',"device":'..v.Device..',"service":"'..v.Service..'","variable":"'..v.Variable..'"}'
	end

	lul_html = lul_html .. ']}'
    return lul_html
end

function saveConfig()
	json = require("json")

	-- Rename the current file
	local fname = dataDir .. DATAMINE_CONFIG
	local frename = dataDir .. DATAMINE_CONFIG .. "." .. os.date() .. ".backup"
	frename = string.gsub(frename, ":", "-")
	ret, err = os.rename(fname, frename)
	if(ret == nil) then
		luup.log(DATAMINE_LOG_NAME .. "Error renaming '"..fname.."' to '" .. frename .. "' --> Error '" .. err)
	end

	local outf = io.open(fname, 'w')
	if(outf == nil) then
		luup.log(DATAMINE_LOG_NAME .. "Unable to open config file for write " .. fname)
	else
		outf:write(json.encode(LogVariables) .. '\n')
		outf:close()
	end

	ChannelCnt = 0
	ChannelRec = 0
	for k,v in pairs (LogVariables.Variables) do
		ChannelCnt = ChannelCnt + 1
		if(v.Logging == 1) then
			ChannelRec = ChannelRec + 1
		end
	end

	luup.variable_set(SERVICE_ID, "ChannelCnt", ChannelCnt, lul_device)
	luup.variable_set(SERVICE_ID, "ChannelRec", ChannelRec, lul_device)
end

-- Get a list of channels that can be graphed
function incomingCtrl(lul_request, lul_parameters, lul_outputformat)
--	luup.log(DATAMINE_LOG_NAME .. "incomingCtrl: " .. lul_request)

	local control  = lul_parameters.control
	local device   = tonumber(lul_parameters.device)
	local service  = lul_parameters.service
	local variable = lul_parameters.variable

	if(control == "list") then
		return controlList(device, service, variable)
	elseif(control == "history") then
		return processHistory(device, service, variable)
	end
end

function controlList(device, service, variable)
	local found = 0

	if(LogVariables == nil) then
		LogVariables = {}
		LogVariables.Variables = {}
	end

	for k,v in pairs (LogVariables.Variables) do
		if(v.Device==device and v.Service==service and v.Variable==variable) then
			if(v.Logging == 1) then
				v.Logging = 0
			else
				v.Logging = 1
			end

			found = 1
		end
	end

	local newTable = {}
	if(found == 0) then
		newTable.Logging  = 1
		newTable.Type     = 0
		if(variable == "Status") then
			newTable.Type = 1
		end
		if(variable == "Watts") then
			newTable.Type = 1
		end
		newTable.Device   = device
		newTable.Service  = service
		newTable.Variable = variable
		newTable.FirstRec = 0
		newTable.Lookup   = {}
		newTable.Archive  = luup.devices[device].description .. " - " .. variable
		newTable.Name     = luup.devices[device].description .. " - " .. variable

		table.insert(LogVariables.Variables, newTable)

		-- Enable logging
		luup.variable_watch('watchVariable', service, variable, tonumber(device))
		luup.log(DATAMINE_LOG_NAME.."Watching: D["..tonumber(device).."] S["..service.."] V["..variable.."]")
	end

	-- Save the configuration
	saveConfig()

	local lul_html = "NOK"
	for k,v in pairs (LogVariables.Variables) do
		if(v.Device==device and v.Service==service and v.Variable==variable) then
			lul_html = '{"list":[{"name":"'..v.Name..'","id":'..k..',"logging":'..v.Logging..',"type":'..v.Type..',"device":'..v.Device..',"service":"'..v.Service..'","variable":"'..v.Variable..'"}]}'
		end
	end

    return lul_html
end

-- Return graph data
function incomingData(lul_request, lul_parameters, lul_outputformat)
--	luup.log(DATAMINE_LOG_NAME .. "incomingData: " .. lul_request)

	local Start      = tonumber(lul_parameters.start)
	local Stop       = tonumber(lul_parameters.stop)
	local Channel
	local ChannelList = {}

	if(lul_parameters.channel1 ~= nul) then
		if(tonumber(lul_parameters.channel1) ~= 0) then
			ChannelList[1] = tonumber(lul_parameters.channel1)
		end
	end
	if(lul_parameters.channel2 ~= nul) then
		if(tonumber(lul_parameters.channel2) ~= 0) then
			ChannelList[2] = tonumber(lul_parameters.channel2)
		end
	end
	if(lul_parameters.channel3 ~= nul) then
		if(tonumber(lul_parameters.channel3) ~= 0) then
			ChannelList[3] = tonumber(lul_parameters.channel3)
		end
	end
	if(lul_parameters.channel4 ~= nul) then
		if(tonumber(lul_parameters.channel4) ~= 0) then
			ChannelList[4] = tonumber(lul_parameters.channel4)
		end
	end

--	for k,v in pairs (Channel) do
--		luup.log(DATAMINE_LOG_NAME.."Graph >> "..k.."<>"..v)
--	end

	local lastVal
	local lastTime
	local nextTime
	local Points
	local first
	local WeekNum
	local logfile
	local inf
	local line
	local Sample1
	local lul_html
	local multiChan
	local minVal
	local maxVal
	local n_val

	lul_html = '{"series":['

	multiChan = 0
	for C,cv in pairs (ChannelList) do
		Channel = cv

		if(Start < LogVariables.Variables[Channel].FirstRec) then
			Start = LogVariables.Variables[Channel].FirstRec
		end

		if(Start < FIRST_YEAR) then
			Start = FIRST_YEAR
		end

		-- Default graph is last 1 day
		if(Stop == 0) then
			Stop = os.time()
			Start = Stop - (86400 * 1)
		end

		-- The end time can't be newer than now. Otherwise we could get into a loop!
		if(Stop > os.time()) then
			Stop = os.time()
		end

		-- Don't try and do anything fancy if the times are screwed up
		-- This shouldn't ever happen!
		if(Stop < Start) then
			Stop = os.time()
			Start = Stop - (86400 * 1)
		end

		-- Calculate the average time between samples
		Sample1 = math.floor((Stop - Start) / 700)

		luup.log(DATAMINE_LOG_NAME .. "ID: "..Channel.." Start: " .. Start .. " Stop: " .. Stop .. " Sample: " .. Sample1)

		if(Sample1 > 43200) then
			WeekNum  = math.floor(Start / LOGTIME_DAILY)
			nextTime = math.floor(Stop  / LOGTIME_DAILY)
			dataType = "D"
		elseif(Sample1 > 1800) then
			WeekNum  = math.floor(Start / LOGTIME_HOURLY)
			nextTime = math.floor(Stop  / LOGTIME_HOURLY)
			dataType = "H"
		else
			-- Needs something in here to NOT sub sample light switch data - at least at a reduced sample!
			if(LogVariables.Variables[Channel].Type == 1) then
	--			Sample1 = 1
			end
			WeekNum  = math.floor(Start / LOGTIME_RAW)
			nextTime = math.floor(Stop  / LOGTIME_RAW)
			dataType = "R"
		end

		-- Find the first record after the start time
		repeat
			logfile = dataDir .. LogVariables.Variables[Channel].Archive .. " ["..dataType..WeekNum.."].txt"
	--		luup.log(DATAMINE_LOG_NAME .. "Trying " .. logfile)
			inf = io.open(logfile, 'r')
			if(inf ~= nil) then
				break
			end
			WeekNum = WeekNum + 1
		until WeekNum > nextTime

		Points   = 0
		first    = 1
		lastVal  = INVALID_VALUE
		lastTime = Start - Sample1
		nextTime = Start
		minVal   =  99999999
		maxVal   = -99999999

		if(multiChan == 1) then
			lul_html = lul_html .. ','
		end
		multiChan = 1

		lul_html = lul_html .. '{"label":"'..LogVariables.Variables[Channel].Name..'","data":['

		if(inf == nil) then
			luup.log(DATAMINE_LOG_NAME .. "1:Unable to open file for read - " .. logfile)
		else
			local l_time = 0
			local l_val  = 0

			while true do
				line = inf:read("*line")
				if(line == nil) then
					-- Close current file
					inf:close()

					-- Need to open the next weekly file
					WeekNum = WeekNum+1
					logfile = dataDir .. LogVariables.Variables[Channel].Archive .. " ["..dataType..WeekNum.."].txt"

					-- Open the file
					inf = io.open(logfile, 'r')
					if(inf == nil) then
						--luup.log(DATAMINE_LOG_NAME .. "2:Unable to open file for read - " .. logfile)
						break
					end

					-- Read the first line
					line = inf:read("*line")
					if(line == nil) then
						break
					end
				end

				local startp,endp = string.find(line,",",1)

				if(startp == nil) then
					luup.log(DATAMINE_LOG_NAME .. "Error reading CSV >> " .. line .. "<<");
					break;
				end

				l_time = tonumber(string.sub(line,1,startp-1))
				l_val  = string.sub(line,startp+1)
				if(LogVariables.Variables[Channel].Alpha == 1) then
--					luup.log(DATAMINE_LOG_NAME .. "Alpha = "..l_val)
					l_val = LogVariables.Variables[Channel].Lookup[l_val]
--					if(l_val ~= nil) then
--						luup.log(DATAMINE_LOG_NAME .. "Alpha = "..l_val)
--					else
--						luup.log(DATAMINE_LOG_NAME .. "Alpha = nil")
--					end
				else
					l_val  = tonumber(l_val)
				end

				if(l_time == nil or l_val == nil) then
					break
				end

				if(l_time > Stop) then
					break
				end

				if(l_time >= nextTime) then
					-- Keep track of max and min values
					if(l_val < minVal) then
						minVal = l_val
					end
					if(l_val > maxVal) then
						maxVal = l_val
					end

					if(first == 0) then
						lul_html = lul_html .. ","

						-- Since we only log changes, we need to assume that the data was
						-- constant between changes. Therefore, to avoid incorrect graphs
						-- we need to add another plot if the time between points is too long.
						lul_html = lul_html .. "["..l_time.."000,"..lastVal.."],"
						Points = Points+1
					else
						if(lastVal ~= INVALID_VALUE) then
							lul_html = lul_html .. "["..Start.."000,"..lastVal.."],"
							lul_html = lul_html .. "["..l_time.."000,"..lastVal.."],"
							Points = 2
						else
							Start = l_time
						end
					end
					first = 0

--					lul_html = lul_html .. "=== "..l_time.."  ["..l_time.."000,"..l_val.."]"
					lul_html = lul_html .. "["..l_time.."000,"..l_val.."]"
					Points = Points+1

					nextTime = nextTime + Sample1

					-- Remember the last datapoint
					-- Required here to avoid issues with subsampling
					lastTime = l_time
					lastVal  = l_val
				elseif(l_time < Start) then
					-- Remember the last datapoint
					-- Required here to get the correct start datapoint
					lastTime = l_time
					lastVal  = l_val
				end
			end

			if(inf ~= nil) then
				-- Close file
				inf:close()
			end
		end

--		if(LogVariables.Variables[Channel].Type == 1) then
			-- If there were no points to display, then we need to add a start point!
			if(Points == 0) then
				lul_html = lul_html .. "["..Start.."000,"..lastVal.."]"
				Points = 1
			end
			lul_html = lul_html .. ",["..Stop.."000,"..lastVal.."]"
			Points   = Points+1
			lastTime = Stop
--		end

		lul_html = lul_html .. ']'

		if(LogVariables.Variables[Channel].Alpha == 1) then
			lul_html = lul_html .. ',"ticks":[';
			first = 1
			for k,v in pairs (LogVariables.Variables[Channel].Lookup) do
				if(first == 0) then
					lul_html = lul_html .. ','
				end
				first = 0
				lul_html = lul_html .. '["'.. k ..'",' .. v .. "]"
			end
			lul_html = lul_html .. "]"
		end
		lul_html = lul_html .. ',"min":'..minVal..',"max":'..maxVal..'}'

	end
	--lul_html = lul_html .. ']'

	lul_html = lul_html .. '],"min":'..Start..',"max":'..lastTime..',"points":'..Points..'}'

    return lul_html
end

local statsTime
local statsVal
local statsChannel
local hourMaxVal
local hourMaxTime
local hourMinVal
local hourMinTime
local hourNext
local dayMaxVal
local dayMaxTime
local dayMinVal
local dayMinTime
local dayNext

--
function dailyHistory(luup_data)
	endTime = math.floor(os.time() / 86400) * 86400
	for k,v in pairs (LogVariables.Variables) do
		if((v.Logging == 1) and (v.LastHistory < endTime)) then
			luup.log(DATAMINE_LOG_NAME.."Processing history START: D["..tonumber(v.Device).."] S["..v.Service.."] V["..v.Variable.."]")
			processDailyRaw(v.Device, v.Service, v.Variable)
			luup.log(DATAMINE_LOG_NAME.."Processing history END: D["..tonumber(v.Device).."] S["..v.Service.."] V["..v.Variable.."]")
		end
	end
end

-- Reprocesses all history data for a variable
-- Deletes the existing stored hourly/daily data
-- and reprocesses from the raw file
function processHistory(device, service, variable)
	local endTime
	local WeekNum
	local logfile
	local inf
	local outf
	local line

--	luup.log(DATAMINE_LOG_NAME .. "START HISTORY ------------------------------------------------------------------------------")
	statsChannel = nil

	for k,v in pairs (LogVariables.Variables) do
		if(v.Device==device and v.Service==service and v.Variable==variable) then
			statsChannel = k;
		end
	end

	if(statsChannel == nil) then
		return "NOK"
	end

	-- Delete all HOURLY and DAILY files
	WeekNum = math.floor(FIRST_YEAR / LOGTIME_HOURLY)
	endTime = math.floor(os.time()  / LOGTIME_HOURLY)
	repeat
		logfile = dataDir .. LogVariables.Variables[statsChannel].Archive .. " [H"..WeekNum.."].txt"
		-- DELETE
		os.remove(logfile)
		WeekNum = WeekNum + 1
	until WeekNum > endTime

	WeekNum = math.floor(FIRST_YEAR / LOGTIME_DAILY)
	endTime = math.floor(os.time()  / LOGTIME_DAILY)
	repeat
		logfile = dataDir .. LogVariables.Variables[statsChannel].Archive .. " [D"..WeekNum.."].txt"
		-- DELETE
		os.remove(logfile)
		WeekNum = WeekNum + 1
	until WeekNum > endTime

	-- Find the first RAW file
	WeekNum = math.floor(FIRST_YEAR / LOGTIME_RAW)
	endTime = math.floor(os.time()  / LOGTIME_RAW)
	repeat
		logfile = dataDir .. LogVariables.Variables[statsChannel].Archive .. " [R"..WeekNum.."].txt"
		--luup.log(DATAMINE_LOG_NAME .. "Trying " .. logfile)
		inf = io.open(logfile, 'r')
		if(inf ~= nil) then
			break
		end
		WeekNum = WeekNum + 1
	until WeekNum > endTime

	-- Reinitialise the time of the first record
	LogVariables.Variables[statsChannel].FirstRec = 0

	statsTime = 0
	statsVal  = 0

	if(inf == nil) then
		luup.log(DATAMINE_LOG_NAME .. "1:Unable to open file for read - " .. logfile)
		return "NOK"
	end

	-- Calculate the time to the last midnight
	endTime = math.floor(os.time() / 86400) * 86400

	while true do
		line = inf:read("*line")
		if(line == nil) then
			-- Close current file
			inf:close()

			-- Need to open the next weekly file
			WeekNum = WeekNum+1
			logfile = dataDir .. LogVariables.Variables[statsChannel].Archive .. " [R"..WeekNum.."].txt"

			luup.log(DATAMINE_LOG_NAME .. logfile)

			-- Open the file
			inf = io.open(logfile, 'r')
			if(inf == nil) then
				luup.log(DATAMINE_LOG_NAME .. "2:Unable to open file for read - " .. logfile)
				break
			end

			-- Read the first line
			line = inf:read("*line")
			if(line == nil) then
				break
			end
		end

		local startp,endp = string.find(line,",",1)

		if(startp == nil) then
			luup.log(DATAMINE_LOG_NAME .. "Error reading CSV >> " .. line .. "<<");
			break
		end

		statsTime = tonumber(string.sub(line,1,startp-1))
		statsVal  = tonumber(string.sub(line,startp+1))

		if(statsTime == nil or statsVal == nil) then
			break
		end

		if(statsTime >= endTime) then
			break
		end

		-- Some initialisation required after we have the first data
		if(LogVariables.Variables[statsChannel].FirstRec == 0) then
			-- Record the first record time
			LogVariables.Variables[statsChannel].FirstRec = statsTime

			-- Initialise the vars
			-- Required here to intialise the dayNext/hourNext vars
			hourMinVal  =  999999
			hourMaxVal  = -999999
			hourNext    = (math.floor(statsTime / 3600) + 1) * 3600
			hourMaxTime = 0
			hourMinTime = 0

			dayMinVal   =  999999
			dayMaxVal   = -999999
			dayNext     = (math.floor(statsTime / 86400) + 1) * 86400
			dayMaxTime  = 0
			dayMinTime  = 0
		end

		-- Process the data
		processStats()
	end

	if(inf ~= nil) then
		-- Close file
		inf:close()
	end

	-- Save the config - saves the history times
	LogVariables.Variables[statsChannel].LastHistory = endTime
	saveConfig()

--	luup.log(DATAMINE_LOG_NAME .. "FINISH HISTORY ------------------------------------------------------------------------------")

    return "OK"
end

-- Processes the raw files
-- Generates the hourly/daily data
function processDailyRaw(device, service, variable)
	local	endTime
	local	WeekNum

	statsChannel = nil
	for k,v in pairs (LogVariables.Variables) do
		if(v.Logging==1 and v.Device==device and v.Service==service and v.Variable==variable) then
			statsChannel = k;
		end
	end

	if(statsChannel == nil) then
		return
	end

	if(LogVariables.Variables[statsChannel].LastHistory == nil) then
		LogVariables.Variables[statsChannel].LastHistory = FIRST_YEAR
	end

	if(LogVariables.Variables[statsChannel].LastHistory < LogVariables.Variables[statsChannel].FirstRec) then
		LogVariables.Variables[statsChannel].LastHistory = LogVariables.Variables[statsChannel].FirstRec
	end

	WeekNum = math.floor(LogVariables.Variables[statsChannel].LastHistory / LOGTIME_RAW)
	endTime = math.floor(os.time()                                        / LOGTIME_RAW)

	-- Find the first record after the start time
	repeat
		logfile = dataDir .. LogVariables.Variables[statsChannel].Archive .. " [R"..WeekNum.."].txt"
		--luup.log(DATAMINE_LOG_NAME .. "Trying:: " .. logfile)
		inf = io.open(logfile, 'r')
		if(inf ~= nil) then
			luup.log(DATAMINE_LOG_NAME .. "FOUND");
			break
		end
		WeekNum = WeekNum + 1
	until WeekNum > endTime

	-- Were any files found?
	if(inf == nil) then
		return
	end

	-- Initialise the vars
	-- Required here to intialise the dayNext/hourNext vars
	hourMinVal  =  999999
	hourMaxVal  = -999999
	hourNext    = (math.floor(LogVariables.Variables[statsChannel].LastHistory / 3600) + 1) * 3600
	hourMaxTime = 0
	hourMinTime = 0

	dayMinVal   =  999999
	dayMaxVal   = -999999
	dayNext     = (math.floor(LogVariables.Variables[statsChannel].LastHistory / 86400) + 1) * 86400
	dayMaxTime  = 0
	dayMinTime  = 0

	startTime = math.floor(LogVariables.Variables[statsChannel].LastHistory / 86400) * 86400
	endTime   = math.floor(os.time()                                        / 86400) * 86400

	-- Don't process more than 7 days data at a time
	--if((endTime - startTime) > (7 * 86400)) then
	--	endTime = startTime + (7 * 86400)
	--end

	-- Now process the whole file
	while true do
		line = inf:read("*line")
		if(line == nil) then
			-- Close current file
			inf:close()

			-- Need to open the next weekly file
			WeekNum = WeekNum+1
			logfile = dataDir .. LogVariables.Variables[statsChannel].Archive .. " [R"..WeekNum.."].txt"

			--luup.log(DATAMINE_LOG_NAME .. "NOW Trying:: " .. logfile)

			-- Open the file
			inf = io.open(logfile, 'r')
			if(inf == nil) then
				luup.log(DATAMINE_LOG_NAME .. "2:Unable to open file for read - " .. logfile)
				break
			end

			-- Read the first line
			line = inf:read("*line")
			if(line == nil) then
				break
			end
		end

		local startp,endp = string.find(line,",",1)

		if(startp == nil) then
			luup.log(DATAMINE_LOG_NAME .. "Error reading CSV >> " .. line .. "<<");
			break
		end

		statsTime = tonumber(string.sub(line,1,startp-1))
		statsVal  = tonumber(string.sub(line,startp+1))

		if(statsTime == nil or statsVal == nil) then
			--luup.log(DATAMINE_LOG_NAME .. "Error 22 " .. line .. "<<");
			break
		end

		if(statsTime >= endTime) then
			--luup.log(DATAMINE_LOG_NAME .. "Error 23 " .. line .. "<<");
			break
		end

		-- Process the data
		if(statsTime >= startTime) then
			processStats()
		end
	end

	if(inf ~= nil) then
		-- Close file
		inf:close()
	end

	-- Save the config - saves the history times
	LogVariables.Variables[statsChannel].LastHistory = endTime
	saveConfig()
end

function processStats()
--	luup.log(statsTime .. "=" .. statsVal .. "  --  ".. hourNext .. ", " .. hourMaxVal .. ", " .. hourMinVal .. ", " .. hourMaxTime .. ", " .. hourMinTime)
	if(statsTime > hourNext) then
		hourNext = math.floor(hourMinTime / LOGTIME_HOURLY)
		logfile = dataDir .. LogVariables.Variables[statsChannel].Archive .. " [H"..hourNext.."].txt"
		outf = io.open(logfile, 'a')
		if(outf == nil) then
			luup.log(DATAMINE_LOG_NAME .. "Unable to open file for write " .. logfile)
		else
			if(hourMaxTime > hourMinTime) then
				outf:write(tostring(hourMinTime) .. ',' .. tostring(hourMinVal) .. '\n')
				outf:write(tostring(hourMaxTime) .. ',' .. tostring(hourMaxVal) .. '\n')
			else
				outf:write(tostring(hourMaxTime) .. ',' .. tostring(hourMaxVal) .. '\n')
				outf:write(tostring(hourMinTime) .. ',' .. tostring(hourMinVal) .. '\n')
			end
		end
		outf:close()

		hourMinVal =  999999
		hourMaxVal = -999999
		hourNext   = (math.floor(statsTime / 3600) + 1) * 3600
	end

	if(statsTime > dayNext) then
		dayNext = math.floor(dayMinTime / LOGTIME_DAILY)
		logfile = dataDir .. LogVariables.Variables[statsChannel].Archive .. " [D"..dayNext.."].txt"
		outf = io.open(logfile, 'a')
		if(outf == nil) then
			luup.log(DATAMINE_LOG_NAME .. "Unable to open file for write " .. logfile)
		else
			if(dayMaxTime > dayMinTime) then
				outf:write(tostring(dayMinTime) .. ',' .. tostring(dayMinVal) .. '\n')
				outf:write(tostring(dayMaxTime) .. ',' .. tostring(dayMaxVal) .. '\n')
			else
				outf:write(tostring(dayMaxTime) .. ',' .. tostring(dayMaxVal) .. '\n')
				outf:write(tostring(dayMinTime) .. ',' .. tostring(dayMinVal) .. '\n')
			end
		end
		outf:close()

		dayMinVal =  999999
		dayMaxVal = -999999
		dayNext   = (math.floor(statsTime / 86400) + 1) * 86400
	end

	if(statsVal > hourMaxVal) then
		hourMaxVal  = statsVal
		hourMaxTime = statsTime
	end
	if(statsVal < hourMinVal) then
		hourMinVal  = statsVal
		hourMinTime = statsTime
	end

	if(statsVal > dayMaxVal) then
		dayMaxVal  = statsVal
		dayMaxTime = statsTime
	end
	if(statsVal < dayMinVal) then
		dayMinVal  = statsVal
		dayMinTime = statsTime
	end
end
